/*!\file Sing.c
 * \brief: implementation of the Sing object
 */

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "stdio.h"
#include <string.h>
#include "../objects.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "../../Container/Container.h"
#include "../../include/include.h"

/*Sing constructors and destructor*/
/*FUNCTION Sing::Sing(){{{1*/
Sing::Sing(){
	this->inputs=NULL;
	this->parameters=NULL;
	return;
}
/*}}}*/
/*FUNCTION Sing::~Sing(){{{1*/
Sing::~Sing(){
	delete inputs;
	this->parameters=NULL;
}
/*}}}*/

/*Object virtual functions definitions: */
/*FUNCTION Sing::copy {{{1*/
Object* Sing::copy() {

	int i;
	Sing* sing=NULL;

	sing=new Sing();

	/*copy fields: */
	sing->id=this->id;
	if(this->inputs){
		sing->inputs=(Inputs*)this->inputs->Copy();
	}
	else{
		sing->inputs=new Inputs();
	}
	/*point parameters: */
	sing->parameters=this->parameters;

	/*pointers: */
	sing->node=this->node;
	sing->matice=this->matice;
	sing->matpar=this->matpar;

	return sing;
}
/*}}}*/
/*FUNCTION Sing::DeepEcho {{{1*/
void Sing::DeepEcho(void){

	printf("Sing:\n");
	printf("   id: %i\n",id);
	node->DeepEcho();
	matice->DeepEcho();
	matpar->DeepEcho();
	printf("   parameters\n");
	parameters->DeepEcho();
	printf("   inputs\n");
	inputs->DeepEcho();

	return;
}
/*}}}*/
/*FUNCTION Sing::Demarshall {{{1*/
void  Sing::Demarshall(char** pmarshalled_dataset){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::Echo{{{1*/

void Sing::Echo(void){

	printf("Sing:\n");
	printf("   id: %i\n",id);
	node->Echo();
	matice->Echo();
	matpar->Echo();
	printf("   parameters\n");
	parameters->Echo();
	printf("   inputs\n");
	inputs->Echo();
}
/*}}}*/
/*FUNCTION Sing::Enum {{{1*/
int Sing::Enum(void){

	return SingEnum;

}
/*}}}*/
/*FUNCTION Sing::Id {{{1*/
int    Sing::Id(void){ return id; }
/*}}}*/
/*FUNCTION Sing::Marshall {{{1*/
void  Sing::Marshall(char** pmarshalled_dataset){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::MashallSize {{{1*/
int   Sing::MarshallSize(){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::MyRank {{{1*/
int    Sing::MyRank(void){ 
	extern int my_rank;
	return my_rank; 
}
/*}}}*/

/*Update virtual functions definitions: */
/*FUNCTION Sing::InputUpdateFromSolution {{{1*/
void  Sing::InputUpdateFromSolution(double* solution){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::InputUpdateFromVector(double* vector, int name, int type);{{{1*/
void  Sing::InputUpdateFromVector(double* vector, int name, int type){

	/*Check that name is an element input*/
	if (!IsInput(name)) return;

	switch(type){

		case VertexEnum:

			/*New SingVertexInpu*/
			double value;

			/*Get values on the 6 vertices*/
			value=vector[node->GetVertexDof()];

			/*update input*/
			this->inputs->AddInput(new SingVertexInput(name,value));
			return;

		default:
			ISSMERROR("type %i (%s) not implemented yet",type,EnumAsString(type));
	}
}
/*}}}*/
/*FUNCTION Sing::InputUpdateFromVector(int* vector, int name, int type);{{{1*/
void  Sing::InputUpdateFromVector(int* vector, int name, int type){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::InputUpdateFromVector(bool* vector, int name, int type);{{{1*/
void  Sing::InputUpdateFromVector(bool* vector, int name, int type){
	ISSMERROR(" not supported yet!");
}
/*}}}*/

/*Element virtual functions definitions: */
/*FUNCTION Sing::ComputeBasalStress {{{1*/
void  Sing::ComputeBasalStress(Vec p_g){

	ISSMERROR("Not implemented yet");

}
/*}}}*/
/*FUNCTION Sing::ComputePressure {{{1*/
void  Sing::ComputePressure(Vec p_g){

	int    dof;
	double pressure;
	double thickness;
	double rho_ice,g;

	/*Get dof list on which we will plug the pressure values: */
	GetDofList1(&dof);

	/*pressure is lithostatic: */
	rho_ice=matpar->GetRhoIce();
	g=matpar->GetG();
	inputs->GetParameterValue(&thickness,ThicknessEnum);
	pressure=rho_ice*g*thickness;
	
	/*plug local pressure values into global pressure vector: */
	VecSetValue(p_g,dof,pressure,INSERT_VALUES);

}
/*}}}*/
/*FUNCTION Sing::ComputeStrainRate {{{1*/
void  Sing::ComputeStrainRate(Vec p_g){

	ISSMERROR("Not implemented yet");

}
/*}}}*/
/*FUNCTION Sing::Configure {{{1*/
void  Sing::Configure(Elements* elementsin,Loads* loadsin, DataSet* nodesin, Materials* materialsin, Parameters* parametersin){

	ISSMERROR(" not supported yet!");

}
/*}}}*/
/*FUNCTION Sing::SetCurrentConfiguration {{{1*/
void  Sing::SetCurrentConfiguration(Elements* elementsin,Loads* loadsin, DataSet* nodesin, Materials* materialsin, Parameters* parametersin){

	ISSMERROR(" not supported yet!");

}
/*}}}*/
/*FUNCTION Sing::CostFunction {{{1*/
double Sing::CostFunction(){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::CreateKMatrix {{{1*/

void  Sing::CreateKMatrix(Mat Kgg){

	int analysis_type;

	/*retrive parameters: */
	parameters->FindParam(&analysis_type,AnalysisTypeEnum);

	/*Just branch to the correct element stiffness matrix generator, according to the type of analysis we are carrying out: */
	if (analysis_type==DiagnosticHutterAnalysisEnum){
		CreateKMatrixDiagnosticHutter( Kgg);
	}
	else{
		ISSMERROR("analysis %i (%s) not supported yet",analysis_type,EnumAsString(analysis_type));
	}

}
/*}}}*/
/*FUNCTION Sing::CreatePVector {{{1*/
void  Sing::CreatePVector(Vec pg){

	int analysis_type;

	/*retrive parameters: */
	parameters->FindParam(&analysis_type,AnalysisTypeEnum);
	
	/*Just branch to the correct load generator, according to the type of analysis we are carrying out: */
	if (analysis_type==DiagnosticHutterAnalysisEnum){
			CreatePVectorDiagnosticHutter( pg);
	}
	else{
		ISSMERROR("analysis %i (%s) not supported yet",analysis_type,EnumAsString(analysis_type));
	}

}
/*}}}*/
/*FUNCTION Sing::Du {{{1*/
void  Sing::Du(Vec){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GetBedList {{{1*/
void  Sing::GetBedList(double*){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GetMatPar {{{1*/
void* Sing::GetMatPar(){

	return matpar;
}
/*}}}*/
/*FUNCTION Sing::GetNodes {{{1*/
void  Sing::GetNodes(void** vpnodes){
	
	Node** pnodes=NULL;

	/*recover nodes: */
	pnodes=(Node**)vpnodes;

	pnodes[0]=node;
}
/*}}}*/
/*FUNCTION Sing::GetOnBed {{{1*/
bool   Sing::GetOnBed(){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GetShelf {{{1*/
bool   Sing::GetShelf(){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GetSolutionFromInputs{{{1*/
void  Sing::GetSolutionFromInputs(Vec solution){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GetThicknessList {{{1*/
void  Sing::GetThicknessList(double* thickness_list){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GetVectorFromInputs{{{1*/
void  Sing::GetVectorFromInputs(Vec vector,int NameEnum){

	int i;
	const int numvertices=1;
	int doflist1[numvertices];

	/*Find NameEnum input in the inputs dataset, and get it to fill in the vector: */
	for(i=0;i<this->inputs->Size();i++){
		Input* input=(Input*)this->inputs->GetObjectByOffset(i);
		if(input->EnumType()==NameEnum){
			/*We found the enum.  Use its values to fill into the vector, using the vertices ids: */
			this->GetDofList1(&doflist1[0]);
			input->GetVectorFromInputs(vector,&doflist1[0]);
			break;
		}
	}
}
/*}}}*/
/*FUNCTION Sing::Gradj {{{1*/
void  Sing::Gradj(Vec gradient,int control_type){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GradB {{{1*/
void  Sing::GradjB(Vec gradient){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::GradjDrag {{{1*/
void  Sing::GradjDrag(Vec gradient){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::InputAXPY{{{1*/
void  Sing::InputAXPY(int YEnum, double scalar, int XEnum){

	Input* xinput=NULL;
	Input* yinput=NULL;

	/*Find x and y inputs: */
	xinput=(Input*)this->inputs->GetInput(XEnum);
	yinput=(Input*)this->inputs->GetInput(YEnum);

	/*some checks: */
	if(!xinput || !yinput)ISSMERROR("%s%s%s%s%s"," input ",EnumAsString(XEnum)," or input ",EnumAsString(YEnum)," could not be found!");
	if(xinput->Enum()!=yinput->Enum())ISSMERROR("%s%s%s%s%s"," input ",EnumAsString(XEnum)," and input ",EnumAsString(YEnum)," are not of the same type!");

	/*Scale: */
	yinput->AXPY(xinput,scalar);
}
/*}}}*/
/*FUNCTION Sing::InputControlConstrain{{{1*/
void  Sing::InputControlConstrain(int control_type, double cm_min, double cm_max){

	Input* input=NULL;

	/*Find input: */
	input=(Input*)this->inputs->GetInput(control_type);
	
	/*Do nothing if we  don't find it: */
	if(!input)return;

	/*Constrain input using cm_min and cm_max: */
	input->Constrain(cm_min,cm_max);

}
/*}}}*/
/*FUNCTION Sing::InputConvergence{{{1*/
void  Sing::InputConvergence(bool* pconverged,double* eps, int* enums,int num_enums,int* criterionenums,double* criterionvalues,int num_criterionenums){

	int i;
	Input** new_inputs=NULL;
	Input** old_inputs=NULL;
	bool    converged=true;

	new_inputs=(Input**)xmalloc(num_enums/2*sizeof(Input*)); //half the enums are for the new inputs
	old_inputs=(Input**)xmalloc(num_enums/2*sizeof(Input*)); //half the enums are for the old inputs
	
	for(i=0;i<num_enums/2;i++){
		new_inputs[i]=(Input*)this->inputs->GetInput(enums[2*i+0]);
		old_inputs[i]=(Input*)this->inputs->GetInput(enums[2*i+1]);
		if(!new_inputs[i])ISSMERROR("%s%s"," could not find input with enum ",EnumAsString(enums[2*i+0]));
		if(!old_inputs[i])ISSMERROR("%s%s"," could not find input with enum ",EnumAsString(enums[2*i+0]));
	}

	/*ok, we've got the inputs (new and old), now loop throught the number of criterions and fill the eps array:*/
	for(i=0;i<num_criterionenums;i++){
		IsInputConverged(eps+i,new_inputs,old_inputs,num_enums/2,criterionenums[i]);
		if(eps[i]>criterionvalues[i]) converged=false; 
	}

	/*Assign output pointers:*/
	*pconverged=converged;

}
/*}}}*/
/*FUNCTION Sing::InputDuplicate{{{1*/
void  Sing::InputDuplicate(int original_enum,int new_enum){

	Input* original=NULL;
	Input* copy=NULL;

	/*Make a copy of the original input: */
	original=(Input*)this->inputs->GetInput(original_enum);
	copy=(Input*)original->copy();

	/*Change copy enum to reinitialized_enum: */
	copy->ChangeEnum(new_enum);

	/*Add copy into inputs, it will wipe off the one already there: */
	inputs->AddObject((Input*)copy);
}
/*}}}*/
/*FUNCTION Sing::InputScale{{{1*/
void  Sing::InputScale(int enum_type,double scale_factor){

	Input* input=NULL;

	/*Make a copy of the original input: */
	input=(Input*)this->inputs->GetInput(enum_type);

	/*Scale: */
	input->Scale(scale_factor);
}
/*}}}*/
/*FUNCTION Sing::InputToResult{{{1*/
void  Sing::InputToResult(int enum_type,int step,double time){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::MassFlux {{{1*/
double Sing::MassFlux( double* segment){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::MaxAbsVx{{{1*/
void  Sing::MaxAbsVx(double* pmaxabsvx, bool process_units){

	int dim;
	double  maxabsvx;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&maxabsvx,VxEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&maxabsvx,1,VxEnum,this->parameters);

	maxabsvx=fabs(maxabsvx);

	/*Assign output pointers:*/
	*pmaxabsvx=maxabsvx;
}
/*}}}*/
/*FUNCTION Sing::MaxAbsVy{{{1*/
void  Sing::MaxAbsVy(double* pmaxabsvy, bool process_units){

	int dim;
	double  maxabsvy;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&maxabsvy,VyEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&maxabsvy,1,VyEnum,this->parameters);
	
	maxabsvy=fabs(maxabsvy);

	/*Assign output pointers:*/
	*pmaxabsvy=maxabsvy;
}
/*}}}*/
/*FUNCTION Sing::MaxAbsVz{{{1*/
void  Sing::MaxAbsVz(double* pmaxabsvz, bool process_units){

	int dim;
	double  maxabsvz;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&maxabsvz,VzEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&maxabsvz,1,VzEnum,this->parameters);

	maxabsvz=fabs(maxabsvz);

	/*Assign output pointers:*/
	*pmaxabsvz=maxabsvz;
}
/*}}}*/
/*FUNCTION Sing::MaxVel{{{1*/
void  Sing::MaxVel(double* pmaxvel, bool process_units){

	int dim;
	double  maxvel;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&maxvel,VelEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&maxvel,1,VelEnum,this->parameters);

	/*Assign output pointers:*/
	*pmaxvel=maxvel;

}
/*}}}*/
/*FUNCTION Sing::MaxVx{{{1*/
void  Sing::MaxVx(double* pmaxvx, bool process_units){

	int dim;
	double  maxvx;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&maxvx,VxEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&maxvx,1,VxEnum,this->parameters);

	/*Assign output pointers:*/
	*pmaxvx=maxvx;

}
/*}}}*/
/*FUNCTION Sing::MaxVy{{{1*/
void  Sing::MaxVy(double* pmaxvy, bool process_units){

	int dim;
	double  maxvy;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&maxvy,VyEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&maxvy,1,VyEnum,this->parameters);

	/*Assign output pointers:*/
	*pmaxvy=maxvy;

}
/*}}}*/
/*FUNCTION Sing::MaxVz{{{1*/
void  Sing::MaxVz(double* pmaxvz, bool process_units){

	int dim;
	double  maxvz;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&maxvz,VzEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&maxvz,1,VzEnum,this->parameters);

	/*Assign output pointers:*/
	*pmaxvz=maxvz;

}
/*}}}*/
/*FUNCTION Sing::MinVel{{{1*/
void  Sing::MinVel(double* pminvel, bool process_units){

	int dim;
	double  minvel;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&minvel,VelEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&minvel,1,VelEnum,this->parameters);

	/*Assign output pointers:*/
	*pminvel=minvel;

}
/*}}}*/
/*FUNCTION Sing::MinVx{{{1*/
void  Sing::MinVx(double* pminvx, bool process_units){

	int dim;
	double  minvx;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&minvx,VxEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&minvx,1,VxEnum,this->parameters);

	/*Assign output pointers:*/
	*pminvx=minvx;

}
/*}}}*/
/*FUNCTION Sing::MinVy{{{1*/
void  Sing::MinVy(double* pminvy, bool process_units){

	int dim;
	double  minvy;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&minvy,VyEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&minvy,1,VyEnum,this->parameters);

	/*Assign output pointers:*/
	*pminvy=minvy;

}
/*}}}*/
/*FUNCTION Sing::MinVz{{{1*/
void  Sing::MinVz(double* pminvz, bool process_units){

	int dim;
	double  minvz;

	/*retrieve dim parameter: */
	parameters->FindParam(&dim,DimEnum);

	/*retrive velocity values at nodes */
	inputs->GetParameterValue(&minvz,VzEnum);

	/*process units if requested: */
	if(process_units)NodalValuesUnitConversion(&minvz,1,VzEnum,this->parameters);

	/*Assign output pointers:*/
	*pminvz=minvz;

}
/*}}}*/
/*FUNCTION Sing::Misfit {{{1*/
double Sing::Misfit(void){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::PatchFill{{{1*/
void  Sing::PatchFill(int* pcount, Patch* patch){
	
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::PatchSize{{{1*/
void  Sing::PatchSize(int* pnumrows, int* pnumvertices,int* pnumnodes){

	ISSMERROR(" not supported yet!");
	
}
/*}}}*/
/*FUNCTION Sing::ProcessResultsUnits{{{1*/
void  Sing::ProcessResultsUnits(void){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::SurfaceArea {{{1*/
double Sing::SurfaceArea( void){
	ISSMERROR(" not supported yet!");
}
/*}}}*/
/*FUNCTION Sing::Update{{{1*/
void Sing::Update(int index, IoModel* iomodel,int analysis_counter,int analysis_type){
	ISSMERROR(" not supported yet!");
}
/*}}}*/

/*Sing specific routines: */
/*FUNCTION Sing::IsInput{{{1*/
bool Sing::IsInput(int name){
	if (name==SurfaceSlopeXEnum ||
				name==SurfaceSlopeYEnum){
		return true;
	}
	else return false;
}
/*}}}*/
/*FUNCTION Sing::CreateKMatrixDiagnosticHutter {{{1*/
void  Sing::CreateKMatrixDiagnosticHutter(Mat Kgg){
	
	const int numgrids=1;
	const int NDOF2=2;
	const int numdofs=numgrids*NDOF2;
	double Ke_gg[numdofs][numdofs]={{1,0},{0,1}};
	int    doflist[numdofs];
	int    numberofdofspernode;
	int    connectivity;

	/*Find connectivity of the node and divide Ke_gg by this connectivity*/
	connectivity=node->GetConnectivity();
	Ke_gg[0][0]=1/(double)connectivity;
	Ke_gg[1][1]=1/(double)connectivity;

	GetDofList(&doflist[0],&numberofdofspernode);

	MatSetValues(Kgg,numdofs,doflist,numdofs,doflist,(const double*)Ke_gg,ADD_VALUES);

}
/*}}}*/
/*FUNCTION Sing::CreatePVectorDiagnosticHutter {{{1*/
void Sing::CreatePVectorDiagnosticHutter( Vec pg){
	
	const int numgrids=1;
	const int NDOF2=2;
	const int numdofs=NDOF2*numgrids;
	int       doflist[numdofs];
	int       dofs[1]={0};
	int       found=0;
	double    slope[2];
	double    slope2;
	double    pe_g[numdofs]={0,0};
	double    ub,vb;
	double    constant_part;
	int       numberofdofspernode;
	double    rho_ice,gravity,n,B;
	double    thickness;
	int       connectivity;

	inputs->GetParameterValue(&slope[0],SurfaceSlopeXEnum);
	inputs->GetParameterValue(&slope[1],SurfaceSlopeYEnum);

	GetDofList(&doflist[0],&numberofdofspernode);

	//Get connectivity of the node
	connectivity=node->GetConnectivity();

	//compute slope2 
	slope2=pow(slope[0],2)+pow(slope[1],2);

	//compute ub
	rho_ice=matpar->GetRhoIce();
	gravity=matpar->GetG();
	n=matice->GetN();
	B=matice->GetB();
	inputs->GetParameterValue(&thickness,ThicknessEnum);

	ub=-1.58*pow((double)10.0,(double)-10.0)*rho_ice*gravity*thickness*slope[0];
	vb=-1.58*pow((double)10.0,(double)-10.0)*rho_ice*gravity*thickness*slope[1];

	//compute constant_part
	constant_part=-2*pow(rho_ice*gravity,n)*pow(slope2,((n-1)/2));

	pe_g[0]=(ub-2.0*pow(rho_ice*gravity,n)*pow(slope2,((n-1)/2.0))*pow(thickness,n)/(pow(B,n)*(n+1))*slope[0])/(double)connectivity;
	pe_g[1]=(vb-2.0*pow(rho_ice*gravity,n)*pow(slope2,((n-1)/2.0))*pow(thickness,n)/(pow(B,n)*(n+1))*slope[1])/(double)connectivity;

	VecSetValues(pg,numdofs,doflist,(const double*)pe_g,ADD_VALUES);

}
/*}}}*/
/*FUNCTION Sing::GetDofList {{{1*/
void  Sing::GetDofList(int* doflist,int* pnumberofdofspernode){

	int i;
	int doflist_per_node[MAXDOFSPERNODE];
	int numberofdofspernode;
	
	node->GetDofList(&doflist_per_node[0],&numberofdofspernode);
	for(i=0;i<numberofdofspernode;i++){
		doflist[i]=doflist_per_node[i];
	}

	/*Assign output pointers:*/
	*pnumberofdofspernode=numberofdofspernode;

}
/*}}}*/
/*FUNCTION Sing::GetDofList1 {{{1*/
void  Sing::GetDofList1(int* doflist){

	int i;
	doflist[0]=node->GetDofList1();

}
/*}}}*/
/*FUNCTION Sing::SetClone {{{1*/
void  Sing::SetClone(int* minranks){

	ISSMERROR("not implemented yet");
}
/*}}}1*/
